@@ -9,31 +9,16 @@
 
 #include "links.h"
 
-int *highlight_positions = NULL;
-int *highlight_lengths = NULL;
-int n_highlight_positions = 0;
+static int *highlight_positions = NULL;
+static int *highlight_lengths = NULL;
+static int n_highlight_positions = 0;
 
 static int root_x = 0;
 static int root_y = 0;
 
-void g_get_search_data(struct f_data *f);
-struct g_object_text * g_find_nearest_object(struct f_data *f, int x, int y);
-int g_find_text_pos(struct g_object_text *, int);
-void g_get_search(struct f_data *, unsigned char *);
-void get_parents_sub(struct g_object *, struct g_object *);
-void g_set_current_link(struct f_data_c *, struct g_object_text *, int, int, int);
-void process_sb_event(struct f_data_c *, int, int);
-void process_sb_move(struct f_data_c *, int);
-static inline int ev_in_rect(struct event *, int, int, int, int);
-int skip_link(struct f_data_c *, int);
-int lr_link(struct f_data_c *, int);
-int g_next_link(struct f_data_c *, int);
-void unset_link(struct f_data_c *);
-void get_searched_sub(struct g_object *, struct g_object *);
-void find_nearest_sub(struct g_object *, struct g_object *);
-void find_next_sub(struct g_object *, struct g_object *);
-void g_find_next_str(struct f_data *);
-
+static void g_get_search_data(struct f_data *f);
+static struct g_object_text * g_find_nearest_object(struct f_data *f, int x, int y);
+static void redraw_link(struct f_data_c *fd, int nl);
 
 static int previous_link=-1;	/* for mouse event handlers */
 
@@ -78,12 +63,12 @@ void g_dummy_mouse(struct f_data_c *fd, struct g_object *a, int x, int y, int b)
 {
 }
 
-int print_all_textarea = 0;
+static unsigned char print_all_textarea = 0;
 
 
 /* returns byte index of x in t->text */
 /* x is relative coordinate within the text (can be out of bounds) */
-int g_find_text_pos(struct g_object_text *t, int x)
+static int g_find_text_pos(struct g_object_text *t, int x)
 {
 	int i=0, p=0;
 	unsigned char *text=t->text;
@@ -157,7 +142,8 @@ void g_text_draw(struct f_data_c *fd, struct g_object_text *t, int x, int y)
 				if (fs->state >= fs->vpos + form->size) fs->vpos = fs->state - form->size + 1;
 				if (fs->state < fs->vpos) fs->vpos = fs->state;
 				*/
-				while ((size_t)fs->vpos < strlen(fs->value) && utf8_diff(fs->value + fs->state, fs->value + fs->vpos) >= form->size) {
+				if ((size_t)fs->vpos > strlen(fs->value)) fs->vpos = strlen(fs->value);
+				while ((size_t)fs->vpos < strlen(fs->value) && textptr_diff(fs->value + fs->state, fs->value + fs->vpos, fd->f_data->opt.cp) >= form->size) {
 					unsigned char *p = fs->value + fs->vpos;
 					FWD_UTF_8(p);
 					fs->vpos = p - fs->value;
@@ -198,8 +184,8 @@ void g_text_draw(struct f_data_c *fd, struct g_object_text *t, int x, int y)
 				}
 				return;
 			case FC_TEXTAREA:
-				cur = _area_cursor(form, fs);
-				if (!(lnx = format_text(fs->value, form->cols, form->wrap))) break;
+				cur = area_cursor(fd, form, fs);
+				if (!(lnx = format_text(fs->value, form->cols, form->wrap, fd->f_data->opt.cp))) break;
 				ln = lnx;
 				yy = y - t->link_order * t->style->height;
 				for (j = 0; j < fs->vypos; j++) if (ln->st) ln++;
@@ -397,11 +383,11 @@ void g_area_draw(struct f_data_c *fd, struct g_object_area *a, int xx, int yy)
 	root_x = xx, root_y = yy;
 	if (!l1) {
 		if (y1 > a->yw) return;
-		else l1 = (struct g_object **)&a->lines[0];
+		else l1 = (struct g_object **)(void *)&a->lines[0];
 	}
 	if (!l2) {
 		if (y2 < 0) return;
-		else l2 = (struct g_object **)&a->lines[a->n_lines - 1];
+		else l2 = (struct g_object **)(void *)&a->lines[a->n_lines - 1];
 	}
 	for (i = l1; i <= l2; i++) {
 		struct g_object *o = *i;
@@ -466,9 +452,9 @@ void get_scrollbar_pos(int dsize, int total, int vsize, int vpos, int *start, in
 	*/
 }
 
-long scroll_bar_frame_color;
-long scroll_bar_area_color;
-long scroll_bar_bar_color;
+static long scroll_bar_frame_color;
+static long scroll_bar_area_color;
+static long scroll_bar_bar_color;
 
 void draw_vscroll_bar(struct graphics_device *dev, int x, int y, int yw, int total, int view, int pos)
 {
@@ -500,7 +486,7 @@ void draw_hscroll_bar(struct graphics_device *dev, int x, int y, int xw, int tot
 	drv->fill_area(dev, x + 2 + epos, y + 2, x + xw - 1, y + G_SCROLL_BAR_WIDTH - 2, scroll_bar_area_color);
 }
 
-void g_get_search(struct f_data *f, unsigned char *s)
+static void g_get_search(struct f_data *f, unsigned char *s)
 {
 	int i;
 	if (!s || !*s) return;
@@ -532,6 +518,11 @@ void g_get_search(struct f_data *f, unsigned char *s)
 	}
 }
 
+static void draw_root(struct f_data_c *scr, int x, int y)
+{
+	scr->f_data->root->draw(scr, scr->f_data->root, x, y);
+}
+
 void draw_graphical_doc(struct terminal *t, struct f_data_c *scr, int active)
 {
 	int r = 0;
@@ -552,17 +543,17 @@ void draw_graphical_doc(struct terminal *t, struct f_data_c *scr, int active)
 		}
 	}
 
-	if (vs->view_pos > scr->f_data->y - scr->yw + scr->f_data->hsb * G_SCROLL_BAR_WIDTH) vs->view_pos = scr->f_data->y - scr->yw + scr->f_data->hsb * G_SCROLL_BAR_WIDTH;
+	if (vs->view_pos > scr->f_data->y - scr->yw + scr->hsb * G_SCROLL_BAR_WIDTH) vs->view_pos = scr->f_data->y - scr->yw + scr->hsb * G_SCROLL_BAR_WIDTH;
 	if (vs->view_pos < 0) vs->view_pos = 0;
-	if (vs->view_posx > scr->f_data->x - scr->xw + scr->f_data->vsb * G_SCROLL_BAR_WIDTH) vs->view_posx = scr->f_data->x - scr->xw + scr->f_data->vsb * G_SCROLL_BAR_WIDTH;
+	if (vs->view_posx > scr->f_data->x - scr->xw + scr->vsb * G_SCROLL_BAR_WIDTH) vs->view_posx = scr->f_data->x - scr->xw + scr->vsb * G_SCROLL_BAR_WIDTH;
 	if (vs->view_posx < 0) vs->view_posx = 0;
 	vx = vs->view_posx;
 	vy = vs->view_pos;
 	restrict_clip_area(t->dev, &old, scr->xp, scr->yp, scr->xp + xw, scr->yp + yw);
-	if (scr->f_data->vsb) draw_vscroll_bar(t->dev, scr->xp + xw - G_SCROLL_BAR_WIDTH, scr->yp, yw - scr->f_data->hsb * G_SCROLL_BAR_WIDTH, scr->f_data->y, yw - scr->f_data->hsb * G_SCROLL_BAR_WIDTH, vs->view_pos);
-	if (scr->f_data->hsb) draw_hscroll_bar(t->dev, scr->xp, scr->yp + yw - G_SCROLL_BAR_WIDTH, xw - scr->f_data->vsb * G_SCROLL_BAR_WIDTH, scr->f_data->x, xw - scr->f_data->vsb * G_SCROLL_BAR_WIDTH, vs->view_posx);
-	if (scr->f_data->vsb && scr->f_data->hsb) drv->fill_area(t->dev, scr->xp + xw - G_SCROLL_BAR_WIDTH, scr->yp + yw - G_SCROLL_BAR_WIDTH, scr->xp + xw, scr->yp + yw, scroll_bar_frame_color);
-	restrict_clip_area(t->dev, NULL, scr->xp, scr->yp, scr->xp + xw - scr->f_data->vsb * G_SCROLL_BAR_WIDTH, scr->yp + yw - scr->f_data->hsb * G_SCROLL_BAR_WIDTH);
+	if (scr->vsb) draw_vscroll_bar(t->dev, scr->xp + xw - G_SCROLL_BAR_WIDTH, scr->yp, yw - scr->hsb * G_SCROLL_BAR_WIDTH, scr->f_data->y, yw - scr->hsb * G_SCROLL_BAR_WIDTH, vs->view_pos);
+	if (scr->hsb) draw_hscroll_bar(t->dev, scr->xp, scr->yp + yw - G_SCROLL_BAR_WIDTH, xw - scr->vsb * G_SCROLL_BAR_WIDTH, scr->f_data->x, xw - scr->vsb * G_SCROLL_BAR_WIDTH, vs->view_posx);
+	if (scr->vsb && scr->hsb) drv->fill_area(t->dev, scr->xp + xw - G_SCROLL_BAR_WIDTH, scr->yp + yw - G_SCROLL_BAR_WIDTH, scr->xp + xw, scr->yp + yw, scroll_bar_frame_color);
+	restrict_clip_area(t->dev, NULL, scr->xp, scr->yp, scr->xp + xw - scr->vsb * G_SCROLL_BAR_WIDTH, scr->yp + yw - scr->hsb * G_SCROLL_BAR_WIDTH);
 	/*debug("buu: %d %d %d, %d %d %d", scr->xl, vx, xw, scr->yl, vy, yw);*/
 	if (drv->flags & GD_DONT_USE_SCROLL) goto rrr;
 	if (scr->xl == -1 || scr->yl == -1) goto rrr;
@@ -581,7 +572,7 @@ void draw_graphical_doc(struct terminal *t, struct f_data_c *scr, int active)
 				struct rect clip1;
 				/*fprintf(stderr, "scroll: %d,%d %d,%d\n", r->x1, r->y1, r->x2, r->y2);*/
 				restrict_clip_area(t->dev, &clip1, r->x1, r->y1, r->x2, r->y2);
-				scr->f_data->root->draw(scr, scr->f_data->root, scr->xp - vs->view_posx, scr->yp - vs->view_pos - (scr->yl - vy));
+				draw_root(scr, scr->xp - vs->view_posx, scr->yp - vs->view_pos - (scr->yl - vy));
 				drv->set_clip_area(t->dev, &clip1);
 			}
 			mem_free(rs);
@@ -598,7 +589,7 @@ void draw_graphical_doc(struct terminal *t, struct f_data_c *scr, int active)
 				struct rect clip1;
 				/*fprintf(stderr, "scroll: %d,%d %d,%d\n", r->x1, r->y1, r->x2, r->y2);*/
 				restrict_clip_area(t->dev, &clip1, r->x1, r->y1, r->x2, r->y2);
-				scr->f_data->root->draw(scr, scr->f_data->root, scr->xp - vs->view_posx, scr->yp - vs->view_pos);
+				draw_root(scr, scr->xp - vs->view_posx, scr->yp - vs->view_pos);
 				drv->set_clip_area(t->dev, &clip1);
 			}
 			mem_free(rs);
@@ -609,30 +600,30 @@ void draw_graphical_doc(struct terminal *t, struct f_data_c *scr, int active)
 		struct rect clip1;
 		if (scr->xl < vx)  {
 			if (scr->yl < vy) {
-				restrict_clip_area(t->dev, &clip1, scr->xp + xw - scr->f_data->vsb * G_SCROLL_BAR_WIDTH - (vx - scr->xl), scr->yp, scr->xp + xw - scr->f_data->vsb * G_SCROLL_BAR_WIDTH, scr->yp + yw - scr->f_data->hsb * G_SCROLL_BAR_WIDTH - (vy - scr->yl));
+				restrict_clip_area(t->dev, &clip1, scr->xp + xw - scr->vsb * G_SCROLL_BAR_WIDTH - (vx - scr->xl), scr->yp, scr->xp + xw - scr->vsb * G_SCROLL_BAR_WIDTH, scr->yp + yw - scr->hsb * G_SCROLL_BAR_WIDTH - (vy - scr->yl));
 			} else {
-				restrict_clip_area(t->dev, &clip1, scr->xp + xw - scr->f_data->vsb * G_SCROLL_BAR_WIDTH - (vx - scr->xl), scr->yp + (scr->yl - vy), scr->xp + xw - scr->f_data->vsb * G_SCROLL_BAR_WIDTH, scr->yp + yw - scr->f_data->hsb * G_SCROLL_BAR_WIDTH);
+				restrict_clip_area(t->dev, &clip1, scr->xp + xw - scr->vsb * G_SCROLL_BAR_WIDTH - (vx - scr->xl), scr->yp + (scr->yl - vy), scr->xp + xw - scr->vsb * G_SCROLL_BAR_WIDTH, scr->yp + yw - scr->hsb * G_SCROLL_BAR_WIDTH);
 			}
 		} else {
 			if (scr->yl < vy) {
-				restrict_clip_area(t->dev, &clip1, scr->xp, scr->yp, scr->xp + (scr->xl - vx), scr->yp + yw - scr->f_data->hsb * G_SCROLL_BAR_WIDTH - (vy - scr->yl));
+				restrict_clip_area(t->dev, &clip1, scr->xp, scr->yp, scr->xp + (scr->xl - vx), scr->yp + yw - scr->hsb * G_SCROLL_BAR_WIDTH - (vy - scr->yl));
 			} else {
-				restrict_clip_area(t->dev, &clip1, scr->xp, scr->yp + (scr->yl - vy), scr->xp + (scr->xl - vx), scr->yp + yw - scr->f_data->hsb * G_SCROLL_BAR_WIDTH);
+				restrict_clip_area(t->dev, &clip1, scr->xp, scr->yp + (scr->yl - vy), scr->xp + (scr->xl - vx), scr->yp + yw - scr->hsb * G_SCROLL_BAR_WIDTH);
 			}
 		}
-		scr->f_data->root->draw(scr, scr->f_data->root, scr->xp - vs->view_posx, scr->yp - vs->view_pos);
+		draw_root(scr, scr->xp - vs->view_posx, scr->yp - vs->view_pos);
 		drv->set_clip_area(t->dev, &clip1);
 		if (scr->yl < vy) {
-			restrict_clip_area(t->dev, NULL, scr->xp, scr->yp + yw - scr->f_data->hsb * G_SCROLL_BAR_WIDTH - (vy - scr->yl), scr->xp + xw - scr->f_data->vsb * G_SCROLL_BAR_WIDTH, scr->yp + yw - scr->f_data->hsb * G_SCROLL_BAR_WIDTH);
+			restrict_clip_area(t->dev, NULL, scr->xp, scr->yp + yw - scr->hsb * G_SCROLL_BAR_WIDTH - (vy - scr->yl), scr->xp + xw - scr->vsb * G_SCROLL_BAR_WIDTH, scr->yp + yw - scr->hsb * G_SCROLL_BAR_WIDTH);
 		} else {
-			restrict_clip_area(t->dev, NULL, scr->xp, scr->yp, scr->xp + xw - scr->f_data->vsb * G_SCROLL_BAR_WIDTH, scr->yp + (scr->yl - vy));
+			restrict_clip_area(t->dev, NULL, scr->xp, scr->yp, scr->xp + xw - scr->vsb * G_SCROLL_BAR_WIDTH, scr->yp + (scr->yl - vy));
 		}
-		scr->f_data->root->draw(scr, scr->f_data->root, scr->xp - vs->view_posx, scr->yp - vs->view_pos);
+		draw_root(scr, scr->xp - vs->view_posx, scr->yp - vs->view_pos);
 	}
 
 	goto eee;
 	rrr:
-	scr->f_data->root->draw(scr, scr->f_data->root, scr->xp - vs->view_posx, scr->yp - vs->view_pos);
+	draw_root(scr, scr->xp - vs->view_posx, scr->yp - vs->view_pos);
 	eee:
 	scr->xl = vx;
 	scr->yl = vy;
@@ -657,15 +648,13 @@ struct draw_data {
 	struct g_object *o;
 };
 
-void draw_one_object_fn(struct terminal *, struct draw_data *);  /* prototype */
-
-void draw_one_object_fn(struct terminal *t, struct draw_data *d)
+static void draw_one_object_fn(struct terminal *t, struct draw_data *d)
 {
 	struct rect clip;
 	struct f_data_c *scr = d->fd;
 	struct g_object *o = d->o;
 	int x, y;
-	restrict_clip_area(t->dev, &clip, scr->xp, scr->yp, scr->xp + scr->xw - scr->f_data->vsb * G_SCROLL_BAR_WIDTH, scr->yp + scr->yw - scr->f_data->hsb * G_SCROLL_BAR_WIDTH);
+	restrict_clip_area(t->dev, &clip, scr->xp, scr->yp, scr->xp + scr->xw - scr->vsb * G_SCROLL_BAR_WIDTH, scr->yp + scr->yw - scr->hsb * G_SCROLL_BAR_WIDTH);
 	get_object_pos(o, &x, &y);
 	o->draw(scr, o, scr->xp - scr->vs->view_posx + x, scr->yp - scr->vs->view_pos + y);
 	drv->set_clip_area(t->dev, &clip);
@@ -705,9 +694,9 @@ void g_line_mouse(struct f_data_c *fd, struct g_object_line *a, int x, int y, in
 	for (i = 0; i < a->n_entries; i++) if (g_forward_mouse(fd, (struct g_object *)a->entries[i], x, y, b)) return;
 }
 
-struct f_data *ffff;
+static struct f_data *ffff;
 
-void get_parents_sub(struct g_object *p, struct g_object *c)
+static void get_parents_sub(struct g_object *p, struct g_object *c)
 {
 	c->parent = p;
 	if (c->get_list) c->get_list(c, get_parents_sub);
@@ -751,10 +740,8 @@ void get_object_pos(struct g_object *o, int *x, int *y)
 	}
 }
 
-void redraw_link(struct f_data_c *fd, int nl);
-
 /* if set_position is 1 sets cursor position in FIELD/AREA elements */
-void g_set_current_link(struct f_data_c *fd, struct g_object_text *a, int x, int y, int set_position)
+static void g_set_current_link(struct f_data_c *fd, struct g_object_text *a, int x, int y, int set_position)
 {
 	if (a->map) {
 		int i;
@@ -791,13 +778,13 @@ void g_set_current_link(struct f_data_c *fd, struct g_object_text *a, int x, int
 				xx=xx<0?0:xx;
 				yy=a->link_order;
 				yy+=fs->vypos;
-				if ((ln = format_text(fs->value, l->form->cols, l->form->wrap))) {
+				if ((ln = format_text(fs->value, l->form->cols, l->form->wrap, fd->f_data->opt.cp))) {
 					int a;
 					for (a = 0; ln[a].st; a++) if (a==yy){
-						int bla=utf8_diff(ln[a].en,ln[a].st);
+						int bla=textptr_diff(ln[a].en,ln[a].st, fd->f_data->opt.cp);
 						
 						fs->state=ln[a].st-fs->value;
-						fs->state = utf8_add(fs->value + fs->state, xx<bla?xx:bla) - fs->value;
+						fs->state = textptr_add(fs->value + fs->state, xx<bla?xx:bla, fd->f_data->opt.cp) - fs->value;
 						break;
 					}
 					mem_free(ln);
@@ -808,7 +795,7 @@ void g_set_current_link(struct f_data_c *fd, struct g_object_text *a, int x, int
 			if (g_char_width(a->style,' ')) {
 				xx=x/g_char_width(a->style,' ');
 			} else xx=x;
-			fs->state=utf8_add(fs->value + ((size_t)fs->vpos > strlen(fs->value) ? strlen(fs->value) : (size_t)fs->vpos), (xx<0?0:xx)) - fs->value;
+			fs->state=textptr_add(fs->value + ((size_t)fs->vpos > strlen(fs->value) ? strlen(fs->value) : (size_t)fs->vpos), (xx<0?0:xx), fd->f_data->opt.cp) - fs->value;
 		}
 	}
 }
@@ -856,10 +843,10 @@ void g_text_mouse(struct f_data_c *fd, struct g_object_text *a, int x, int y, in
 	}
 }
 
-void process_sb_event(struct f_data_c *fd, int off, int h)
+static void process_sb_event(struct f_data_c *fd, int off, int h)
 {
 	int spos, epos;
-	int w = h ? fd->f_data->hsbsize : fd->f_data->vsbsize;
+	int w = h ? fd->hsbsize : fd->vsbsize;
 	get_scrollbar_pos(w - 4, h ? fd->f_data->x : fd->f_data->y, w, h ? fd->vs->view_posx : fd->vs->view_pos, &spos, &epos);
 	spos += 2;
 	epos += 2;
@@ -870,19 +857,19 @@ void process_sb_event(struct f_data_c *fd, int off, int h)
 		fd->ses->scrolloff = off - spos - 1;
 		return;
 	}
-	if (off < spos) if (h) fd->vs->view_posx -= fd->xw - fd->f_data->vsb * G_SCROLL_BAR_WIDTH;
-			else fd->vs->view_pos -= fd->yw - fd->f_data->hsb * G_SCROLL_BAR_WIDTH;
-	else if (h) fd->vs->view_posx += fd->xw - fd->f_data->vsb * G_SCROLL_BAR_WIDTH;
-		else fd->vs->view_pos += fd->yw - fd->f_data->hsb * G_SCROLL_BAR_WIDTH;
+	if (off < spos) if (h) fd->vs->view_posx -= fd->xw - fd->vsb * G_SCROLL_BAR_WIDTH;
+			else fd->vs->view_pos -= fd->yw - fd->hsb * G_SCROLL_BAR_WIDTH;
+	else if (h) fd->vs->view_posx += fd->xw - fd->vsb * G_SCROLL_BAR_WIDTH;
+		else fd->vs->view_pos += fd->yw - fd->hsb * G_SCROLL_BAR_WIDTH;
 	fd->vs->orig_view_pos = fd->vs->view_pos;
 	fd->vs->orig_view_posx = fd->vs->view_posx;
 	draw_graphical_doc(fd->ses->term, fd, 1);
 }
 
-void process_sb_move(struct f_data_c *fd, int off)
+static void process_sb_move(struct f_data_c *fd, int off)
 {
 	int h = fd->ses->scrolltype;
-	int w = h ? fd->f_data->hsbsize : fd->f_data->vsbsize;
+	int w = h ? fd->hsbsize : fd->vsbsize;
 	int rpos = off - 2 - fd->ses->scrolloff;
 	int st, en;
 	get_scrollbar_pos(w - 4, h ? fd->f_data->x : fd->f_data->y, w, h ? fd->vs->view_posx : fd->vs->view_pos, &st, &en);
@@ -905,32 +892,32 @@ static inline int ev_in_rect(struct event *ev, int x1, int y1, int x2, int y2)
 int is_link_in_view(struct f_data_c *fd, int nl)
 {
 	struct link *l = &fd->f_data->links[nl];
-	return fd->vs->view_pos < l->r.y2 && fd->vs->view_pos + fd->f_data->opt.yw - fd->f_data->hsb * G_SCROLL_BAR_WIDTH > l->r.y1;
+	return fd->vs->view_pos < l->r.y2 && fd->vs->view_pos + fd->yw - fd->hsb * G_SCROLL_BAR_WIDTH > l->r.y1;
 }
 
-int skip_link(struct f_data_c *fd, int nl)
+static int skip_link(struct f_data_c *fd, int nl)
 {
 	struct link *l = &fd->f_data->links[nl];
 	return !l->where && !l->form;
 }
 
-void redraw_link(struct f_data_c *fd, int nl)
+static void redraw_link(struct f_data_c *fd, int nl)
 {
 	struct link *l = &fd->f_data->links[nl];
 	struct rect r;
 	memcpy(&r, &l->r, sizeof(struct rect));
-	r.x1 += fd->f_data->opt.xp - fd->vs->view_posx;
-	r.x2 += fd->f_data->opt.xp - fd->vs->view_posx;
-	r.y1 += fd->f_data->opt.yp - fd->vs->view_pos;
-	r.y2 += fd->f_data->opt.yp - fd->vs->view_pos;
+	r.x1 += fd->xp - fd->vs->view_posx;
+	r.x2 += fd->xp - fd->vs->view_posx;
+	r.y1 += fd->yp - fd->vs->view_pos;
+	r.y2 += fd->yp - fd->vs->view_pos;
 	t_redraw(fd->ses->term->dev, &r);
 }
 
-int lr_link(struct f_data_c *fd, int nl)
+static int lr_link(struct f_data_c *fd, int nl)
 {
 	struct link *l = &fd->f_data->links[nl];
 	int xx = fd->vs->view_posx;
-	if (l->r.x2 > fd->vs->view_posx + fd->xw - fd->f_data->vsb * G_SCROLL_BAR_WIDTH) fd->vs->view_posx = l->r.x2 - (fd->xw - fd->f_data->vsb * G_SCROLL_BAR_WIDTH);
+	if (l->r.x2 > fd->vs->view_posx + fd->xw - fd->vsb * G_SCROLL_BAR_WIDTH) fd->vs->view_posx = l->r.x2 - (fd->xw - fd->vsb * G_SCROLL_BAR_WIDTH);
 	if (l->r.x1 < fd->vs->view_posx) fd->vs->view_posx = l->r.x1;
 	fd->vs->orig_view_posx = fd->vs->view_posx;
 	return xx != fd->vs->view_posx;
@@ -949,7 +936,7 @@ int g_next_link(struct f_data_c *fd, int dir)
 	if (n < 0 || n >= fd->f_data->nlinks) {
 		if (r == 1) {
 			fd->vs->current_link = -1;
-			if (fd->vs->view_pos > fd->f_data->y - fd->yw + fd->f_data->hsb * G_SCROLL_BAR_WIDTH) fd->vs->view_pos = fd->f_data->y - fd->yw + fd->f_data->hsb * G_SCROLL_BAR_WIDTH;
+			if (fd->vs->view_pos > fd->f_data->y - fd->yw + fd->hsb * G_SCROLL_BAR_WIDTH) fd->vs->view_pos = fd->f_data->y - fd->yw + fd->hsb * G_SCROLL_BAR_WIDTH;
 			if (fd->vs->view_pos < 0) fd->vs->view_pos = 0;
 			if (orig_link != -1 && is_link_in_view(fd, orig_link)) fd->vs->current_link = orig_link;
 			fd->vs->orig_link = fd->vs->current_link;
@@ -961,11 +948,11 @@ int g_next_link(struct f_data_c *fd, int dir)
 				fd->vs->orig_view_pos = fd->vs->view_pos;
 				return 0;
 			}
-			fd->vs->view_pos -= fd->f_data->opt.yw - fd->f_data->hsb * G_SCROLL_BAR_WIDTH;
+			fd->vs->view_pos -= fd->yw - fd->hsb * G_SCROLL_BAR_WIDTH;
 			fd->vs->orig_view_pos = fd->vs->view_pos;
 		} else {
-			if (fd->vs->view_pos >= fd->f_data->y - fd->yw + fd->f_data->hsb * G_SCROLL_BAR_WIDTH) return 0;
-			fd->vs->view_pos += fd->f_data->opt.yw - fd->f_data->hsb * G_SCROLL_BAR_WIDTH;
+			if (fd->vs->view_pos >= fd->f_data->y - fd->yw + fd->hsb * G_SCROLL_BAR_WIDTH) return 0;
+			fd->vs->view_pos += fd->yw - fd->hsb * G_SCROLL_BAR_WIDTH;
 			fd->vs->orig_view_pos = fd->vs->view_pos;
 		}
 		r = 1;
@@ -993,7 +980,7 @@ int g_next_link(struct f_data_c *fd, int dir)
 	return r;
 }
 
-void unset_link(struct f_data_c *fd)
+static void unset_link(struct f_data_c *fd)
 {
 	int n = fd->vs->current_link;
 	fd->vs->current_link = -1;
@@ -1007,9 +994,8 @@ void unset_link(struct f_data_c *fd)
 
 int g_frame_ev(struct session *ses, struct f_data_c *fd, struct event *ev)
 {
-	int x, y;
 	if (!fd->f_data) return 0;
-	switch (ev->ev) {
+	switch ((int)ev->ev) {
 		case EV_MOUSE:
 			if ((ev->b & BM_BUTT) == B_WHEELUP) goto up;
 			if ((ev->b & BM_BUTT) == B_WHEELDOWN) goto down;
@@ -1019,8 +1005,6 @@ int g_frame_ev(struct session *ses, struct f_data_c *fd, struct event *ev)
 			if ((ev->b & BM_BUTT) == B_WHEELRIGHT) goto right;
 			if ((ev->b & BM_BUTT) == B_WHEELLEFT1) goto left1;
 			if ((ev->b & BM_BUTT) == B_WHEELRIGHT1) goto right1;
-			x = ev->x;
-			y = ev->y;
 			if ((ev->b & BM_ACT) == B_MOVE) ses->scrolling = 0;
 			if (ses->scrolling == 1) process_sb_move(fd, ses->scrolltype ? ev->x : ev->y);
 			if (ses->scrolling == 2) {
@@ -1042,16 +1026,16 @@ int g_frame_ev(struct session *ses, struct f_data_c *fd, struct event *ev)
 			}
 
 	
-			if ((ev->b & BM_ACT) == B_DOWN && fd->f_data->vsb && ev_in_rect(ev, fd->xw - G_SCROLL_BAR_WIDTH, 0, fd->xw, fd->yw - fd->f_data->hsb * G_SCROLL_BAR_WIDTH)) {
+			if ((ev->b & BM_ACT) == B_DOWN && fd->vsb && ev_in_rect(ev, fd->xw - G_SCROLL_BAR_WIDTH, 0, fd->xw, fd->yw - fd->hsb * G_SCROLL_BAR_WIDTH)) {
 				process_sb_event(fd, ev->y, 0);
 				break;
 			}
-			if ((ev->b & BM_ACT) == B_DOWN && fd->f_data->hsb && ev_in_rect(ev, 0, fd->yw - G_SCROLL_BAR_WIDTH, fd->xw - fd->f_data->vsb * G_SCROLL_BAR_WIDTH, fd->yw)) {
+			if ((ev->b & BM_ACT) == B_DOWN && fd->hsb && ev_in_rect(ev, 0, fd->yw - G_SCROLL_BAR_WIDTH, fd->xw - fd->vsb * G_SCROLL_BAR_WIDTH, fd->yw)) {
 				process_sb_event(fd, ev->x, 1);
 				break;
 			}
-			if (fd->f_data->vsb && ev_in_rect(ev, fd->xw - G_SCROLL_BAR_WIDTH, 0, fd->xw, fd->yw)) return 0;
-			if (fd->f_data->hsb && ev_in_rect(ev, 0, fd->yw - G_SCROLL_BAR_WIDTH, fd->xw, fd->yw)) return 0;
+			if (fd->vsb && ev_in_rect(ev, fd->xw - G_SCROLL_BAR_WIDTH, 0, fd->xw, fd->yw)) return 0;
+			if (fd->hsb && ev_in_rect(ev, 0, fd->yw - G_SCROLL_BAR_WIDTH, fd->xw, fd->yw)) return 0;
 
 			if ((ev->b & BM_ACT) == B_DOWN && (ev->b & BM_BUTT) == B_MIDDLE) {
 				scrll:
@@ -1158,19 +1142,19 @@ int g_frame_ev(struct session *ses, struct f_data_c *fd, struct event *ev)
 				struct link *l;
 				if (fd->vs->current_link >= 0 && fd->vs->current_link < fd->f_data->nlinks) {
 					l = &fd->f_data->links[fd->vs->current_link];
-					set_window_ptr(ses->win, fd->f_data->opt.xp + l->r.x1 - fd->vs->view_posx, fd->f_data->opt.yp + l->r.y1 - fd->vs->view_pos);
+					set_window_ptr(ses->win, fd->xp + l->r.x1 - fd->vs->view_posx, fd->yp + l->r.y1 - fd->vs->view_pos);
 				} else {
-					set_window_ptr(ses->win, fd->f_data->opt.xp, fd->f_data->opt.yp);
+					set_window_ptr(ses->win, fd->xp, fd->yp);
 				}
 				return enter(ses, fd, 0);
 			}
 			if (ev->x == KBD_PAGE_DOWN || (ev->x == ' ' && !(ev->y & KBD_ALT)) || (upcase(ev->x) == 'F' && ev->y & KBD_CTRL)) {
 				unset_link(fd);
-				if (fd->vs->view_pos == fd->f_data->y - fd->yw + fd->f_data->hsb * G_SCROLL_BAR_WIDTH) {
+				if (fd->vs->view_pos == fd->f_data->y - fd->yw + fd->hsb * G_SCROLL_BAR_WIDTH) {
 					fd->vs->orig_view_pos = fd->vs->view_pos;
 					return 0;
 				}
-				fd->vs->view_pos += fd->f_data->opt.yw - fd->f_data->hsb * G_SCROLL_BAR_WIDTH;
+				fd->vs->view_pos += fd->yw - fd->hsb * G_SCROLL_BAR_WIDTH;
 				fd->vs->orig_view_pos = fd->vs->view_pos;
 				return 3;
 			}
@@ -1185,13 +1169,13 @@ int g_frame_ev(struct session *ses, struct f_data_c *fd, struct event *ev)
 					fd->vs->orig_view_pos = fd->vs->view_pos;
 					return 0;
 				}
-				fd->vs->view_pos -= fd->f_data->opt.yw - fd->f_data->hsb * G_SCROLL_BAR_WIDTH;
+				fd->vs->view_pos -= fd->yw - fd->hsb * G_SCROLL_BAR_WIDTH;
 				fd->vs->orig_view_pos = fd->vs->view_pos;
 				return 3;
 			}
 			if (0) {
 				down:
-				if (fd->vs->view_pos == fd->f_data->y - fd->yw + fd->f_data->hsb * G_SCROLL_BAR_WIDTH) {
+				if (fd->vs->view_pos == fd->f_data->y - fd->yw + fd->hsb * G_SCROLL_BAR_WIDTH) {
 					fd->vs->orig_view_pos = fd->vs->view_pos;
 					return 0;
 				}
@@ -1210,7 +1194,7 @@ int g_frame_ev(struct session *ses, struct f_data_c *fd, struct event *ev)
 				return 3;
 			}
 			if (ev->x == KBD_DEL || (upcase(ev->x) == 'N' && ev->y & KBD_CTRL)) {
-				if (fd->vs->view_pos == fd->f_data->y - fd->yw + fd->f_data->hsb * G_SCROLL_BAR_WIDTH) {
+				if (fd->vs->view_pos == fd->f_data->y - fd->yw + fd->hsb * G_SCROLL_BAR_WIDTH) {
 					fd->vs->orig_view_pos = fd->vs->view_pos;
 					return 0;
 				}
@@ -1229,7 +1213,7 @@ int g_frame_ev(struct session *ses, struct f_data_c *fd, struct event *ev)
 			}
 			if (/*ev->x == KBD_DOWN*/ 0) {
 				down1:
-				if (fd->vs->view_pos == fd->f_data->y - fd->yw + fd->f_data->hsb * G_SCROLL_BAR_WIDTH) {
+				if (fd->vs->view_pos == fd->f_data->y - fd->yw + fd->hsb * G_SCROLL_BAR_WIDTH) {
 					fd->vs->orig_view_pos = fd->vs->view_pos;
 					return 0;
 				}
@@ -1265,7 +1249,7 @@ int g_frame_ev(struct session *ses, struct f_data_c *fd, struct event *ev)
 			}
 			if (ev->x == ']') {
 				right:
-				if (fd->vs->view_posx == fd->f_data->x - fd->xw + fd->f_data->vsb * G_SCROLL_BAR_WIDTH) {
+				if (fd->vs->view_posx == fd->f_data->x - fd->xw + fd->vsb * G_SCROLL_BAR_WIDTH) {
 					fd->vs->orig_view_posx = fd->vs->view_posx;
 					return 0;
 				}
@@ -1285,7 +1269,7 @@ int g_frame_ev(struct session *ses, struct f_data_c *fd, struct event *ev)
 			}
 			if (/*ev->x == KBD_RIGHT*/ 0) {
 				right1:
-				if (fd->vs->view_posx == fd->f_data->x - fd->xw + fd->f_data->vsb * G_SCROLL_BAR_WIDTH) {
+				if (fd->vs->view_posx == fd->f_data->x - fd->xw + fd->vsb * G_SCROLL_BAR_WIDTH) {
 					fd->vs->orig_view_posx = fd->vs->view_posx;
 					return 0;
 				}
@@ -1366,9 +1350,9 @@ void draw_title(struct f_data_c *f)
 	mem_free(title);
 }
 
-struct f_data *srch_f_data;
+static struct f_data *srch_f_data;
 
-void get_searched_sub(struct g_object *p, struct g_object *c)
+static void get_searched_sub(struct g_object *p, struct g_object *c)
 {
 	if (c->draw == (void (*)(struct f_data_c *, struct g_object *, int, int))g_text_draw) {
 		struct g_object_text *t = (struct g_object_text *)c;
@@ -1383,7 +1367,7 @@ void get_searched_sub(struct g_object *p, struct g_object *c)
 	}
 }
 
-void g_get_search_data(struct f_data *f)
+static void g_get_search_data(struct f_data *f)
 {
 	int i;
 	srch_f_data = f;
@@ -1416,7 +1400,7 @@ static inline int dist_to_rect(int x, int y, int x1, int y1, int x2, int y2)
 	return w;
 }
 
-void find_nearest_sub(struct g_object *p, struct g_object *c)
+static void find_nearest_sub(struct g_object *p, struct g_object *c)
 {
 	if (fnd_found) return;
 	/*printf("object: x=%d, y=%d\n", c->x, c->y);*/
@@ -1452,7 +1436,7 @@ void find_nearest_sub(struct g_object *p, struct g_object *c)
 	
 }
 
-struct g_object_text * g_find_nearest_object(struct f_data *f, int x, int y)
+static struct g_object_text * g_find_nearest_object(struct f_data *f, int x, int y)
 {
 	fnd_obj=NULL;
 	fnd_x=x;
@@ -1463,18 +1447,18 @@ struct g_object_text * g_find_nearest_object(struct f_data *f, int x, int y)
 	return fnd_obj;
 }
 
-unsigned char *search_word;
+static unsigned char *search_word;
 
-int find_refline;
-int find_direction;
+static int find_refline;
+static int find_direction;
 
-int find_opt_yy;
-int find_opt_y;
-int find_opt_yw;
-int find_opt_x;
-int find_opt_xw;
+static int find_opt_yy;
+static int find_opt_y;
+static int find_opt_yw;
+static int find_opt_x;
+static int find_opt_xw;
 
-void find_next_sub(struct g_object *p, struct g_object *c)
+static void find_next_sub(struct g_object *p, struct g_object *c)
 {
 	if (c->draw == (void (*)(struct f_data_c *, struct g_object *, int, int))g_text_draw) {
 		struct g_object_text *t = (struct g_object_text *)c;
@@ -1490,14 +1474,13 @@ void find_next_sub(struct g_object *p, struct g_object *c)
 			if (yy < find_refline) yy += MAXINT / 2;
 			if (find_direction < 0) yy = MAXINT - yy;
 			if (find_opt_yy == -1 || yy > find_opt_yy) {
-				int i, l, ll;
+				int i, l;
 				find_opt_yy = yy;
 				find_opt_y = y;
 				find_opt_yw = t->style->height;
 				find_opt_x = x;
 				find_opt_xw = t->xw;
 				l = strlen(t->text);
-				ll = strlen(search_word);
 				for (i = 0; i < l; i++) {
 					unsigned char *tt;
 					if (!(compare_case_utf8(t->text + i, search_word))) goto no_ch;
@@ -1516,7 +1499,7 @@ void find_next_sub(struct g_object *p, struct g_object *c)
 	if (c->get_list) c->get_list(c, find_next_sub);
 }
 
-void g_find_next_str(struct f_data *f)
+static void g_find_next_str(struct f_data *f)
 {
 	find_opt_yy = -1;
 	if (f->root && f->root->get_list) f->root->get_list(f->root, find_next_sub);
@@ -1527,7 +1510,7 @@ void g_find_next(struct f_data_c *f, int a)
 	g_get_search_data(f->f_data);
 	g_get_search(f->f_data, f->ses->search_word);
 	search_word = f->ses->search_word;
-	if (!f->f_data->n_search_positions) msg_box(f->ses->term, NULL, TEXT(T_SEARCH), AL_CENTER, TEXT(T_SEARCH_STRING_NOT_FOUND), NULL, 1, TEXT(T_CANCEL), NULL, B_ENTER | B_ESC);
+	if (!f->f_data->n_search_positions) msg_box(f->ses->term, NULL, TEXT_(T_SEARCH), AL_CENTER, TEXT_(T_SEARCH_STRING_NOT_FOUND), NULL, 1, TEXT_(T_CANCEL), NULL, B_ENTER | B_ESC);
 
 	highlight_positions = f->f_data->search_positions;
 	highlight_lengths = f->f_data->search_lengths;
@@ -1535,7 +1518,7 @@ void g_find_next(struct f_data_c *f, int a)
 
 	if ((!a && f->ses->search_direction == -1) ||
 	     (a && f->ses->search_direction == 1)) find_refline = f->vs->view_pos;
-	else find_refline = f->vs->view_pos + f->yw - f->f_data->hsb * G_SCROLL_BAR_WIDTH;
+	else find_refline = f->vs->view_pos + f->yw - f->hsb * G_SCROLL_BAR_WIDTH;
 	find_direction = -f->ses->search_direction;
 
 	g_find_next_str(f->f_data);
@@ -1545,12 +1528,12 @@ void g_find_next(struct f_data_c *f, int a)
 	n_highlight_positions = 0;
 
 	if (find_opt_yy == -1) goto d;
-	if (!a || find_opt_y < f->vs->view_pos || find_opt_y + find_opt_yw >= f->vs->view_pos + f->yw - f->f_data->hsb * G_SCROLL_BAR_WIDTH) {
-		f->vs->view_pos = find_opt_y - (f->yw - f->f_data->hsb * G_SCROLL_BAR_WIDTH) / 2;
+	if (!a || find_opt_y < f->vs->view_pos || find_opt_y + find_opt_yw >= f->vs->view_pos + f->yw - f->hsb * G_SCROLL_BAR_WIDTH) {
+		f->vs->view_pos = find_opt_y - (f->yw - f->hsb * G_SCROLL_BAR_WIDTH) / 2;
 		f->vs->orig_view_pos = f->vs->view_pos;
 	}
-	if (find_opt_x < f->vs->view_posx || find_opt_x + find_opt_xw >= f->vs->view_posx + f->xw - f->f_data->vsb * G_SCROLL_BAR_WIDTH) {
-		f->vs->view_posx = find_opt_x + find_opt_xw / 2 - (f->xw - f->f_data->vsb * G_SCROLL_BAR_WIDTH) / 2;
+	if (find_opt_x < f->vs->view_posx || find_opt_x + find_opt_xw >= f->vs->view_posx + f->xw - f->vsb * G_SCROLL_BAR_WIDTH) {
+		f->vs->view_posx = find_opt_x + find_opt_xw / 2 - (f->xw - f->vsb * G_SCROLL_BAR_WIDTH) / 2;
 		f->vs->orig_view_posx = f->vs->view_posx;
 	}
 
